import functools
import inspect
from importlib import import_module

from django.conf import settings
from six import iteritems

try:
    from django.utils.module_loading import import_string
except ImportError:
    def import_string(value):
        module_name, func_name = value.rsplit('.', 1)
        module = import_module(module_name)
        return getattr(module, func_name)


def create_report(request=None):
    """Run all checks and return a tuple containing results and boolea to
    indicate to indicate if all things are healthy.

    """
    report = {}
    has_error = False

    for service, check_func in _get_check_functions(request=request):
        report[service] = check_func() or False

        if not report[service]:
            has_error = True
    return report, not has_error


def create_service_result(service, request=None):
    functions = list(_get_check_functions(only=[service], request=request))
    if not functions:
        return

    check_func = functions[0][1]
    return check_func() or False


def _get_check_functions(only=None, request=None):
    checks = _get_registered_health_checks()
    for service, func_string in iteritems(checks):
        if only and service not in only:
            continue

        check_func = import_string(func_string)

        spec = inspect.getargspec(check_func)
        if spec.args == ['request']:
            check_func = functools.partial(check_func, request)

        yield service, check_func


def _get_registered_health_checks():
    return getattr(settings, 'HEALTH_CHECKS', {})
